library jaguar_auth.authenticator;

import 'dart:async';
import 'package:jaguar_auth/src/hasher/hasher.dart';

import 'package:jaguar_common/jaguar_common.dart';

export 'package:jaguar_common/jaguar_common.dart';

/// Specifies an interface that authenticators and authorizers use
/// to manage [UserModel]
abstract class AuthModelManager {
  /// Returns user [UserModel] by given [authenticationId]
  Future<UserBase> fetchModelByAuthenticationId(String authenticationId);

  /// Returns user [UserModel] by given [authorizationId]
  Future<UserBase> fetchModelByAuthorizationId(String authorizationId);

  /// Checks password [keyword] for given userId [authId] and
  /// returns the user [UserModel] if passwords match. Returns [null] if the
  /// password doesn't match or user with given authId is not found.
  Future<UserBase> authenticate(String authId, String keyword);
}

/// Model manager to authenticate against a static list of user models
class WhiteListPasswordChecker implements AuthModelManager {
  /// User models to white list
  final Map<String, UserModel> models;

  /// Password hasher
  final Hasher hasher;

  const WhiteListPasswordChecker(Map<String, UserModel> models, {Hasher hasher})
      : models = models ?? const {},
        hasher = hasher ?? const NoHasher();

  Future<UserBase> authenticate(String username, String password) async {
    UserModel model = await fetchModelByAuthenticationId(username);

    if (model == null) {
      return null;
    }

    if (!hasher.verify(password, model.loginPassword)) {
      return null;
    }

    return model;
  }

  Future<UserBase> fetchModelByAuthenticationId(String authName) async =>
      models.values.firstWhere((model) => model.loginId == authName,
          orElse: () => null);

  Future<UserBase> fetchModelByAuthorizationId(String sessionId) async {
    if (!models.containsKey(sessionId)) {
      return null;
    }

    return models[sessionId];
  }
}
